/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import java.util.Iterator;

import de.uni_paderborn.fujaba.uml.UMLDiagramItem;
import de.upb.tools.fca.*;



/**
 * An Actor of a {@link UsecaseDiagram}.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.17 $
 * @see       UsecaseDiagram
 */
public class UsecaseActor extends UMLDiagramItem
{
   /**
    * Constructor for class UsecaseActor
    */
   public UsecaseActor()
   {
      setName ("Actor " + getID());
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public String toString()
   {
      return getName();
   }



   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String name = "";


   /**
    * Get the name attribute of the UsecaseActor object
    *
    * @return   The name value
    */
   public String getName()
   {
      return name;
   }


   /**
    * Set the value of name.
    *
    * @param name  Value to assign to name.
    */
   public void setName (String name)
   {
      if ( (this.name == null && name != null) ||
          (this.name != null && !this.name.equals (name)))
      {
         String oldValue = this.name;
         this.name = name;
         firePropertyChange ("name", oldValue, this.name);
      }
   }



   /**
    * <pre>
    *               n    usages     1
    * UsecaseUsage ------------------- UsecaseActor
    *               usages      actor
    * </pre>
    */

   private FHashSet usages;


   /**
    * UMLMethod: '+ hasInUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean hasInUsages (UsecaseUsage value)
   {
      return  ( (this.usages != null) &&
          (value != null) &&
         this.usages.contains (value));
   }


   /**
    * UMLMethod: '+ iteratorOfUsages () : Iterator'
    *
    * @return   No description provided
    */
   public Iterator iteratorOfUsages()
   {
      return  ( (this.usages == null)
         ? FEmptyIterator.get()
         : this.usages.iterator());
   }


   /**
    * UMLMethod: '+ sizeOfUsages () : Integer'
    *
    * @return   No description provided
    */
   public int sizeOfUsages()
   {
      return  ( (this.usages == null)
         ? 0
         : this.usages.size());
   }


   /**
    * UMLMethod: '+ addToUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  The object added.
    * @return       No description provided
    */
   public boolean addToUsages (UsecaseUsage value)
   {
      boolean changed = false;
      if (value != null)
      {
         if (this.usages == null)
         {
            this.usages = new FPropHashSet (this, "usages"); // or FTreeSet () or FLinkedList ()

         }
         changed = this.usages.add (value);
         if (changed)
         {
            value.setActor (this);
         }
      }
      return changed;
   }


   /**
    * UMLMethod: '+ removeFromUsages (value : UsecaseUsage) : Boolean'
    *
    * @param value  No description provided
    * @return       No description provided
    */
   public boolean removeFromUsages (UsecaseUsage value)
   {
      boolean changed = false;
      if ( (this.usages != null) &&  (value != null))
      {
         changed = this.usages.remove (value);
         if (changed)
         {
            value.setActor (null);
         }
      }
      return changed;
   }


   /**
    * UMLMethod: '+ removeAllFromUsages () : Void'
    */
   public void removeAllFromUsages()
   {
      UsecaseUsage tmpValue;
      Iterator iter = this.iteratorOfUsages();
      while (iter.hasNext())
      {
         tmpValue = (UsecaseUsage) iter.next();
         this.removeFromUsages (tmpValue);
      }
   }



   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      Iterator iter = iteratorOfUsages();
      while (iter.hasNext())
      {
         UsecaseUsage usecaseUsage = (UsecaseUsage) iter.next();
         usecaseUsage.removeYou();
      }
      removeAllFromUsages();
      super.removeYou();
   }
}

/*
 * $Log: UsecaseActor.java,v $
 * Revision 1.17  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
