/*
 * The FUJABA ToolSuite project:
 *
 *   FUJABA is the acronym for 'From Uml to Java And Back Again'
 *   and originally aims to provide an environment for round-trip
 *   engineering using UML as visual programming language. During
 *   the last years, the environment has become a base for several
 *   research activities, e.g. distributed software, database
 *   systems, modelling mechanical and electrical systems and
 *   their simulation. Thus, the environment has become a project,
 *   where this source code is part of. Further details are avail-
 *   able via http://www.fujaba.de
 *
 *      Copyright (C) 1997-2004 Fujaba Development Group
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA 02111-1307, USA or download the license under
 *   http://www.gnu.org/copyleft/lesser.html
 *
 * WARRANTY:
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *   GNU Lesser General Public License for more details.
 *
 * Contact adress:
 *
 *   Fujaba Management Board
 *   Software Engineering Group
 *   University of Paderborn
 *   Warburgerstr. 100
 *   D-33098 Paderborn
 *   Germany
 *
 *   URL  : http://www.fujaba.de
 *   email: info@fujaba.de
 *
 */
package de.uni_paderborn.fujaba.usecase;

import de.uni_paderborn.fujaba.asg.ASGElement;
import de.uni_paderborn.fujaba.uml.UMLDiagramItem;


/**
 * The {@link UsecaseDiagram}'s System-box. It uses a {@link UsecaseSystemDiagram} to hold
 * all the {@link Usecase}s.
 *
 * @author    $Author: schneider $
 * @version   $Revision: 1.20 $
 * @see       Usecase
 * @see       UsecaseDiagram
 * @see       UsecaseSystemDiagram
 */
public class UsecaseSystem extends UMLDiagramItem
{

   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private void init()
   {
      UsecaseSystemDiagram usecaseSystemDiagram = new UsecaseSystemDiagram ("diagram", null);
      setSystemDiagram (usecaseSystemDiagram);
   }


   /**
    * Constructor for class UsecaseSystem
    */
   public UsecaseSystem()
   {
      super();
      init();
   }


   /**
    * Constructor for class UsecaseSystem
    *
    * @param name  No description provided
    * @param diag  No description provided
    */
   public UsecaseSystem (String name, UsecaseDiagram diag)
   {
      super();
      setName (name);
      diag.addToElements (this);
      init();
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    *
    * @return   No description provided
    */
   public String toString()
   {
      return getName();
   }


   /**
    * No comment provided by developer, please add a comment to improve documentation.
    */
   private String name = "";


   /**
    * Get the name attribute of the UsecaseSystem object
    *
    * @return   The name value
    */
   public String getName()
   {
      return name;
   } // getName


   /**
    * Set the value of name.
    *
    * @param name  Value to assign to name.
    */
   public void setName (String name)
   {
      if ( (this.name == null && name != null) ||
          (this.name != null && !this.name.equals (name)))
      {
         String oldValue = this.name;

         this.name = name;

         firePropertyChange ("name", oldValue, this.name);
      }
   }


   /**
    * UMLMethod: '+ removeYou () : Void'
    */
   public void removeYou()
   {
      UsecaseSystemDiagram tmpSystemDiagram = getSystemDiagram();
      if (tmpSystemDiagram != null)
      {
         setSystemDiagram (null);
      } // if

   }


   /**
    * <pre>
    *                       1         assoc         1
    * UsecaseSystemDiagram --------------------------- UsecaseSystem
    *                       systemDiagram      system
    * </pre>
    */
   private UsecaseSystemDiagram systemDiagram;


   /**
    * UMLMethod: '+ setSystemDiagram (value : UsecaseSystemDiagram) : Boolean'
    *
    * @param value  The new systemDiagram value
    * @return       No description provided
    */
   public boolean setSystemDiagram (UsecaseSystemDiagram value)
   {
      boolean changed = false;
      if (this.systemDiagram != value)
      {
         UsecaseSystemDiagram oldValue = this.systemDiagram;
         if (this.systemDiagram != null)
         {
            this.systemDiagram = null;
            oldValue.setSystem (null);
         }
         this.systemDiagram = value;
         if (value != null)
         {
            value.setSystem (this);
         }
         changed = true;
         firePropertyChange ("systemDiagram", oldValue, value);
      }
      return changed;
   }


   /**
    * UMLMethod: '+ getSystemDiagram () : UsecaseSystemDiagram'
    *
    * @return   The systemDiagram value
    */
   public UsecaseSystemDiagram getSystemDiagram()
   {
      return this.systemDiagram;
   }


   /**
    * Searches the ASG tree for a given id
    *
    * @param id  The id to search for.
    * @return    The Element with the given id, null if not found.
    */
   public ASGElement searchID (String id)
   {
      ASGElement elem = super.searchID (id);
      UsecaseSystemDiagram systemDiag = getSystemDiagram();
      if ( (elem == null) &&  (systemDiag != null))
      {
         elem = systemDiag.searchID (id);
      }
      return elem;
   }
}

/*
 * $Log: UsecaseSystem.java,v $
 * Revision 1.20  2004/10/20 17:50:28  schneider
 * Introduction of interfaces for class diagram classes
 *
 */
